package com.m2pool.chat.service.impl;

import com.m2pool.chat.constant.Destination;
import com.m2pool.chat.dto.WebsocketMessageDto;
import com.m2pool.chat.entity.ChatMessage;
import com.m2pool.chat.entity.ChatRoom;
import com.m2pool.chat.entity.StompPrincipal;
import com.m2pool.chat.mapper.ChatMessageMapper;
import com.m2pool.chat.mapper.ChatRoomMapper;
import com.m2pool.chat.service.StompService;
import com.m2pool.chat.vo.UserMessageVo;
import com.m2pool.common.core.web.Result.AjaxResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.messaging.simp.user.SimpUserRegistry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Date;

import static com.m2pool.chat.constant.UserType.CUSTOMER;
import static com.m2pool.chat.constant.UserType.TOURIST;

@Service
public class StompServiceImpl implements StompService {

    @Autowired
    private SimpUserRegistry userRegistry;

    @Autowired
    private SimpMessagingTemplate messagingTemplate;

    @Resource
    private ChatMessageMapper chatMessageMapper;

    @Resource
    private ChatRoomMapper chatRoomMapper;

    @Resource
    private TransactionTemplate transactionTemplate;

    @Override
    public AjaxResult sendMessageToUser(StompPrincipal principal, UserMessageVo userMessageVo) {
        // 检查目标用户是否在线
        if (!checkOnline(userMessageVo)) {
            return AjaxResult.error("目标用户不在线");
        }

        WebsocketMessageDto build = WebsocketMessageDto.builder()
                .type(userMessageVo.getType())
                .content(userMessageVo.getContent())
                .sendEmail(principal.getName())
                .sendUserType(principal.getType())
                .sendTime(new Date())
                .build();
        if(!TOURIST.equals(userMessageVo.getReceiveUserType())){
            build.setRoomId(userMessageVo.getRoomId());
        }
        //当前用户发送其他人, 发送给指定用户. 否则默认发送给当前发送者
        messagingTemplate.convertAndSendToUser(userMessageVo.getEmail(), Destination.QUEUE + "/" + userMessageVo.getEmail(),build);
        // 接收者和发送者 都不是游客才能存储消息 和修改聊天室信息。后续可改为消息中间件解耦形式。
        if (!TOURIST.equals(userMessageVo.getReceiveUserType()) && !TOURIST.equals(principal.getType())){

            transactionTemplate.execute(new TransactionCallbackWithoutResult() {
                @Override
                protected void doInTransactionWithoutResult(TransactionStatus status) {
                    try {
                        // 插入消息并获取 ID
                        insertMessage(principal,userMessageVo);
                        // 更新房间信息
                        updateRoom(userMessageVo.getRoomId(),principal.getType());
                    } catch (Exception e) {
                        // 回滚事务
                        status.setRollbackOnly();
                        throw e;
                    }
                }
            });
        }
        //可以用void
        return AjaxResult.success("成功");
    }

    /**
     * 检查用户是否在线
     * @return
     */
    private boolean checkOnline(UserMessageVo userMessageVo){
        return userRegistry.getUsers().stream()
                .anyMatch(user -> user.getName().equals(userMessageVo.getEmail()));
    }

    /**
     * 新增消息
     * @param userMessageVo
     * @return
     */
    private Long insertMessage(StompPrincipal principal,UserMessageVo userMessageVo){
        ChatMessage message = ChatMessage.builder()
                .sendEmail(principal.getName())
                .content(userMessageVo.getContent())
                .type(userMessageVo.getType())
                .roomId(userMessageVo.getRoomId())
                .build();
        chatMessageMapper.insert(message);
        return message.getId();
    }

    /**
     * 修改聊天信息
     * @param roomId
     * @param sendUserType
     */
    private void updateRoom(Long roomId,Integer sendUserType){
        ChatRoom room = ChatRoom.builder().id(roomId).build();
        if (CUSTOMER.equals(sendUserType)) {
            room.setLastCustomerSendTime(LocalDateTime.now());
        } else {
            room.setLastUserSendTime(LocalDateTime.now());
        }
        chatRoomMapper.updateById(room);
    }
}